@@ -33,6 +33,7 @@
 #include <sdf/sdf.hh>
 
 #include <ignition/math/Rand.hh>
+#include "ignition/common/Profiler.hh"
 
 #include "gazebo/gazebo.hh"
 #include "gazebo/transport/transport.hh"
@@ -100,6 +101,9 @@ namespace gazebo
 
     /// \brief Save argv for access by system plugins.
     char **systemPluginsArgv;
+
+    /// \brief Set whether to lockstep physics and rendering
+    bool lockstep = false;
   };
 }
 
@@ -149,6 +153,7 @@ bool Server::ParseArgs(int _argc, char **_argv)
     ("verbose", "Increase the messages written to the terminal.")
     ("help,h", "Produce this help message.")
     ("pause,u", "Start the server in a paused state.")
+    ("lockstep", "Lockstep simulation so sensor update rates are respected.")
     ("physics,e", po::value<std::string>(),
      "Specify a physics engine (ode|bullet|dart|simbody).")
     ("play,p", po::value<std::string>(), "Play a log file.")
@@ -282,6 +287,12 @@ bool Server::ParseArgs(int _argc, char **_argv)
     }
   }
 
+  if (this->dataPtr->vm.count("lockstep"))
+  {
+    this->dataPtr->lockstep = true;
+  }
+  rendering::set_lockstep_enabled(this->dataPtr->lockstep);
+
   if (!this->PreLoad())
   {
     gzerr << "Unable to load gazebo\n";
@@ -497,7 +508,11 @@ bool Server::LoadImpl(sdf::ElementPtr _elem,
       << " seconds for namespaces. Giving up.\n";
   }
 
-  physics::init_worlds(rendering::update_scene_poses);
+  if (this->dataPtr->lockstep)
+    physics::init_worlds(rendering::update_scene_poses);
+  else
+    physics::init_worlds(nullptr);
+
   this->dataPtr->stop = false;
 
   return true;
@@ -542,8 +557,8 @@ void Server::Run()
     std::cerr << "sigaction(2) failed while setting up for SIGINT" << std::endl;
 
   // The following was added in
-  // https://bitbucket.org/osrf/gazebo/pull-requests/2923, but it is causing
-  // shutdown issues when gazebo is used with ros.
+  // https://osrf-migration.github.io/gazebo-gh-pages/#!/osrf/gazebo/pull-requests/2923,
+  // but it is causing shutdown issues when gazebo is used with ros.
   // if (sigaction(SIGTERM, &sigact, NULL))
   // {
   //   std::cerr << "sigaction(15) failed while setting up for SIGTERM"
@@ -582,18 +597,37 @@ void Server::Run()
 
   this->dataPtr->initialized = true;
 
+  IGN_PROFILE_THREAD_NAME("gzserver");
   // Stay on this loop until Gazebo needs to be shut down
   // The server and sensor manager outlive worlds
   while (!this->dataPtr->stop)
   {
+    IGN_PROFILE("Server::Run");
+    IGN_PROFILE_BEGIN("ProcessControlMsgs");
+    if (this->dataPtr->lockstep)
+      rendering::wait_for_render_request("", 0.100);
+    // bool ret = rendering::wait_for_render_request("", 0.100);
+    // if (ret == false)
+    //   gzerr << "time out reached!" << std::endl;
+
     this->ProcessControlMsgs();
+    IGN_PROFILE_END();
 
     if (physics::worlds_running())
+    {
+      IGN_PROFILE_BEGIN("run_once");
       sensors::run_once();
+      IGN_PROFILE_END();
+    }
     else if (sensors::running())
+    {
+      IGN_PROFILE_BEGIN("stop");
       sensors::stop();
+      IGN_PROFILE_END();
+    }
 
-    common::Time::MSleep(1);
+    if (!this->dataPtr->lockstep)
+      common::Time::MSleep(1);
   }
 
   // Shutdown gazebo
